@@ -30,13 +30,13 @@ from kfp.dsl._container_op import BaseOp
 from kfp_tekton.compiler import __tekton_api_version__ as tekton_api_version
 
 
-RESOURCE_OP_IMAGE = ":".join(["aipipeline/kubectl-wrapper", "latest"])
+RESOURCE_OP_IMAGE = ":".join(["quay.io/aipipeline/kubectl-wrapper", "latest"])
 TEKTON_HOME_RESULT_PATH = "/tekton/home/tep-results/"
 
 # The image to use in basic bash steps such as copying results in multi-step.
 TEKTON_BASH_STEP_IMAGE = 'busybox'
 TEKTON_COPY_RESULTS_STEP_IMAGE = 'library/bash'
-GENERATE_COMPONENT_SPEC_ANNOTATIONS = True
+GENERATE_COMPONENT_SPEC_ANNOTATIONS = env.get('GENERATE_COMPONENT_SPEC_ANNOTATIONS', True)
 
 
 def _get_base_step(name: str):
@@ -157,6 +157,9 @@ def _get_resourceOp_template(op: BaseOp,
             ],
             'steps': [
                 {
+                    "command": [
+                        "kubeclient"
+                    ],
                     "args": [
                         "--action=$(params.action)",
                         "--merge-strategy=$(params.merge-strategy)",
@@ -394,7 +397,8 @@ def _process_base_ops(op: BaseOp):
 
 def _op_to_template(op: BaseOp,
                     pipelinerun_output_artifacts={},
-                    artifact_items={}):
+                    artifact_items={},
+                    generate_component_spec_annotations=True):
     """Generate template given an operator inherited from BaseOp."""
 
     # Display name
@@ -553,7 +557,7 @@ def _op_to_template(op: BaseOp,
                                                                             for volume in processed_op.volumes]
         template['spec']['volumes'].sort(key=lambda x: x['name'])
 
-    if isinstance(op, dsl.ContainerOp) and op._metadata and GENERATE_COMPONENT_SPEC_ANNOTATIONS:
+    if isinstance(op, dsl.ContainerOp) and op._metadata and GENERATE_COMPONENT_SPEC_ANNOTATIONS and generate_component_spec_annotations:
         component_spec_dict = op._metadata.to_dict()
         component_spec_digest = hashlib.sha256(json.dumps(component_spec_dict, sort_keys=True).encode()).hexdigest()
         component_name = component_spec_dict.get('name', op.name)
@@ -652,4 +656,24 @@ def _op_to_template(op: BaseOp,
             if artifact[0] not in verified_result_size_map[step_counter].keys():
                 artifact[1] = '%s%s' % (TEKTON_HOME_RESULT_PATH, sanitize_k8s_name(artifact[0], allow_capital=True))
                 artifact_items[op.name][i] = artifact
+    # Add workspaces into task spec if defined as annotations
+    workspace_map = "{}"
+    if processed_op.pod_annotations:
+        workspace_map = processed_op.pod_annotations.get("workspaces", "{}")
+    if workspace_map and workspace_map != "{}":
+        try:
+            workspace_map = json.loads(workspace_map)
+        except ValueError:
+            raise ("workspace_map annotation is not a valid JSON")
+        workspaces = template['spec'].get("workspaces", [])
+        for key, value in workspace_map.items():
+            workspace_item = {"name": key}
+            if value.get("readOnly", None) is not None:
+                workspace_item["readOnly"] = value["readOnly"]
+            if value.get("mountPath", None) is not None:
+                workspace_item["mountPath"] = value["mountPath"]
+            workspaces.append(workspace_item)
+        template['spec']["workspaces"] = workspaces
+        processed_op.pod_annotations.pop("workspaces")
+        template['metadata']['annotations'].pop("workspaces")
     return template
