@@ -27,6 +27,8 @@ import collections
 from os import environ as env
 from typing import Callable, List, Text, Dict, Any
 import hashlib
+from kubernetes.client.models import V1Volume
+from kubernetes import client
 
 import yaml
 # Kubeflow Pipeline imports
@@ -145,6 +147,10 @@ class TektonCompiler(Compiler):
     self.security_context = None
     self.automount_service_account_token = None
     self.group_names = {}
+    self.pipeline_env = {}
+    self.pipeline_workspaces = {}
+    self.task_workspaces = {}
+    self.generate_component_spec_annotations = True
     super().__init__(**kwargs)
 
   def _set_pipeline_conf(self, tekton_pipeline_conf: TektonPipelineConf):
@@ -157,6 +163,9 @@ class TektonCompiler(Compiler):
     self.resource_in_separate_yaml = tekton_pipeline_conf.resource_in_separate_yaml
     self.security_context = tekton_pipeline_conf.security_context
     self.automount_service_account_token = tekton_pipeline_conf.automount_service_account_token
+    self.pipeline_env = tekton_pipeline_conf.pipeline_env
+    self.pipeline_workspaces = tekton_pipeline_conf.pipeline_workspaces
+    self.generate_component_spec_annotations = tekton_pipeline_conf.generate_component_spec_annotations
 
   def _resolve_value_or_reference(self, value_or_reference, potential_references):
     """_resolve_value_or_reference resolves values and PipelineParams, which could be task parameters or input parameters.
@@ -603,7 +612,8 @@ class TektonCompiler(Compiler):
 
     op_to_steps_handler = op_to_templates_handler or (lambda op: [_op_to_template(op,
                                                                   self.output_artifacts,
-                                                                  self.artifact_items)])
+                                                                  self.artifact_items,
+                                                                  self.generate_component_spec_annotations)])
     root_group = pipeline.groups[0]
 
     # Call the transformation functions before determining the inputs/outputs, otherwise
@@ -1016,6 +1026,12 @@ class TektonCompiler(Compiler):
             ],
             'taskSpec': template['spec'],
           }
+        if template['spec'].get("workspaces"):
+          workspaces_spec = []
+          for item in template['spec']['workspaces']:
+            workspaces_spec.append({"name": item["name"], "workspace": item["name"]})
+            self.task_workspaces[item["name"]] = True
+          task_ref['workspaces'] = workspaces_spec
 
         for i in template['spec'].get('steps', []):
           # TODO: change the below conditions to map with a label
@@ -1110,8 +1126,10 @@ class TektonCompiler(Compiler):
           task_annotations = template['metadata'].get('annotations', {})
 
           # Updata default metadata at the end.
-          task_ref['taskSpec']['metadata']['labels'] = task_labels
-          task_ref['taskSpec']['metadata']['annotations'] = task_annotations
+          if task_labels:
+            task_ref['taskSpec']['metadata']['labels'] = task_labels
+          if task_annotations:
+            task_ref['taskSpec']['metadata']['annotations'] = task_annotations
 
         task_refs.append(task_ref)
 
@@ -1380,19 +1398,27 @@ class TektonCompiler(Compiler):
     self.pipeline_annotations['pipelines.kubeflow.org/big_data_passing_format'] = BIG_DATA_PATH_FORMAT
 
     if self.pipeline_annotations:
-      pipeline_run['metadata']['annotations'] = pipeline_run['metadata'].setdefault('annotations', {})
+      pipeline_run['metadata'].setdefault('annotations', {})
       pipeline_run['metadata']['annotations'].update(self.pipeline_annotations)
 
+    def python_name_to_yaml_name(name: str):
+      return re.sub(r'_([a-z])', lambda x: x.group(1).upper(), name)
+
     if self.security_context:
-      pipeline_run['spec']['podTemplate'] = pipeline_run['spec'].get('podTemplate', {})
+      pipeline_run['spec'].setdefault('podTemplate', {})
       for key, value in self.security_context.to_dict().items():
         if value is not None:
           pipeline_run['spec']['podTemplate']['securityContext'] = \
             pipeline_run['spec']['podTemplate'].setdefault('securityContext', {})
-          pipeline_run['spec']['podTemplate']['securityContext'][key] = value
+          pipeline_run['spec']['podTemplate']['securityContext'][python_name_to_yaml_name(key)] = value
     if self.automount_service_account_token is not None:
-      pipeline_run['spec']['podTemplate'] = pipeline_run['spec'].get('podTemplate', {})
+      pipeline_run['spec'].setdefault('podTemplate', {})
       pipeline_run['spec']['podTemplate']['automountServiceAccountToken'] = self.automount_service_account_token
+    if self.pipeline_env:
+      pipeline_run['spec'].setdefault('podTemplate', {})
+      pipeline_run['spec']['podTemplate'].setdefault('env', [])
+      for key, value in self.pipeline_env.items():
+        pipeline_run['spec']['podTemplate']['env'].append({'name': key, 'value': value})
 
     # Generate TaskRunSpec PodTemplate:s
     task_run_spec = []
@@ -1616,6 +1642,27 @@ class TektonCompiler(Compiler):
     if pipeline_conf and pipeline_conf.data_passing_method is not None:
       workflow = fix_big_data_passing_using_volume(workflow, pipeline_conf)
 
+    # Inject user defined pipeline level workspaces
+    if self.pipeline_workspaces:
+      workflow['spec'].setdefault('workspaces', [])
+      workflow['spec']["pipelineSpec"].setdefault('workspaces', [])
+      for key, value in self.pipeline_workspaces.items():
+        workspaceSpec = {"name": key}
+        if isinstance(value[0], V1Volume):
+          workspaceSpec = client.ApiClient().sanitize_for_serialization(value[0])
+          workspaceSpec["name"] = key
+        else:
+          workspaceSpec['volumeClaimTemplate'] = {"spec": client.ApiClient().sanitize_for_serialization(value[0])}
+        if value[1]:
+          workspaceSpec['subPath'] = value[1]
+        workflow['spec']['workspaces'].append(workspaceSpec)
+        workflow['spec']["pipelineSpec"]['workspaces'].append({"name": workspaceSpec["name"]})
+    
+    workspace_list = [workspace['name'] for workspace in workflow['spec']["pipelineSpec"].get("workspaces", [])]
+    for key in self.task_workspaces:
+      if key not in workspace_list:
+        raise ValueError("Missing workspace %s in the Tekton pipeline config object." % key)
+
     if pipeline_conf and pipeline_conf.timeout > 0:
       workflow['spec'].setdefault('timeouts', {'pipeline': '0s', 'tasks': '0s'})
       workflow['spec']['timeouts']['tasks'] = '%ds' % pipeline_conf.timeout
@@ -1930,13 +1977,13 @@ def _validate_workflow(workflow: Dict[Text, Any]):
   def _find_items(obj, search_key, current_path="", results_dict=dict()) -> dict:
     if isinstance(obj, dict):
       if search_key in obj:
-        results_dict.update({"%s.%s" % (current_path, search_key): obj[search_key]})
+        results_dict.update({("%s.%s" % (current_path, search_key)).lstrip("."): obj[search_key]})
       for k, v in obj.items():
         _find_items(v, search_key, "%s.%s" % (current_path, k), results_dict)
     elif isinstance(obj, list):
       for i, list_item in enumerate(obj):
         _find_items(list_item, search_key, "%s[%i]" % (current_path, i), results_dict)
-    return {k.lstrip("."): v for k, v in results_dict.items()}
+    return results_dict
 
   non_k8s_names = {path: name for path, name in _find_items(workflow, "name").items()
                    if "metadata" in path and name != sanitize_k8s_name(name, max_length=253)
